// Description								/*{{{*/
// $Id: indexfile.h,v 1.2 2002/07/25 18:07:18 niemeyer Exp $
/* ######################################################################

   Index File - Abstraction for an index of archive/source file.

   There are 3 primary sorts of index files, all represented by this
   class:

   Binary index files
   Bianry index files decribing the local system
   Source index files

   They are all bundled together here, and the interfaces for
   sources.list, acquire, cache gen and record parsing all use this class
   to acess the underlying representation.

   ##################################################################### */
									/*}}}*/
#ifndef PKGLIB_INDEXFILE_H
#define PKGLIB_INDEXFILE_H

#include <string>
#include <apt-pkg/pkgcache.h>
#include <apt-pkg/srcrecords.h>
#include <apt-pkg/pkgrecords.h>

using std::string;

class pkgAcquire;
class pkgCacheGenerator;
class OpProgress;
class pkgIndexFile
{
   public:

   class Type
   {
      public:

      // Global list of Items supported
      static Type **GlobalList;
      static unsigned long GlobalListLen;
      static Type *GetType(const char *Type);

      const char *Label;

      virtual pkgRecords::Parser *CreatePkgParser(pkgCache::PkgFileIterator /*File*/) const {return 0;}
      Type();
   };

   virtual const Type *GetType() const = 0;

   // Return descriptive strings of various sorts
   virtual string ArchiveInfo(pkgCache::VerIterator Ver) const;
   virtual string SourceInfo(pkgSrcRecords::Parser const &Record,
			     pkgSrcRecords::File const &File) const;
   virtual string Describe(bool Short) const = 0;
   /* a virtual method with default parameter is confusing; instead, define: */
   string Describe() const { return Describe(false); }

   // Interface for acquire
   virtual string ArchiveURI(const string &/*File*/) const {return string();}
   virtual bool GetIndexes(pkgAcquire *Owner) const;

   // CNC:2002-07-04
   // The API has been extended in ALT for updating the in-memory cache
   // (namely, the contents of the pkgRepository objects stored in RepList
   // in rpmindexfile.cc; see https://bugzilla.altlinux.org/41816 ).
   virtual bool InvalidateReleases() const = 0; // the extension made in ALT
   virtual bool GetReleases(pkgAcquire *Owner) const = 0;

   // Interface for the record parsers
   virtual pkgSrcRecords::Parser *CreateSrcParser() const {return 0;}

   // Interface for the Cache Generator
   virtual bool Exists() const = 0;
   virtual bool HasPackages() const = 0;
   virtual unsigned long Size() const = 0;
   virtual bool Merge(pkgCacheGenerator &/*Gen*/,OpProgress &/*Prog*/) const = 0;
   virtual bool MergeFileProvides(pkgCacheGenerator &/*Gen*/,OpProgress &/*Prog*/) const = 0;
   virtual pkgCache::PkgFileIterator FindInCache(pkgCache &Cache) const;

   virtual ~pkgIndexFile() {}
};

#endif
